
#####################################################################
# LASSPentry sets up nice bindings for an entry - Russ Thompson
#   e is a window name
#   cmd is a tcl command to be executed whenever the Return button 
#      is pressed.  Should return 0 for no error or 1 for an error. 
#      Since the cmd is put directly into the binding, the percent 
#      notation can be used.  For example, to get the value of the 
#      entry, use:  [%W get]   
#      for numeric data only, use: {catch {expr -[%W get]}}
#      for integers only, use:     {catch {expr ~[%W get]}}
#      for numbers >0, use:        {expr {[catch {expr -[%W get]}] || !([%W get]>0)}}
#   All of the standard entry configuration options work as one might expect.
proc LASSPentry {e cmd args} {
    eval entry $e $args
    
    bind $e  <Any-Key> {
	if {"%A" != ""} {
	    %W configure -fg red -selectforeground red
	}
    }
    bind $e  <Key-BackSpace> {
	%W configure -fg red -selectforeground red
    }
    bind $e  <Key-Delete> [bind $e <Key-BackSpace>]

    #handle the default case for $cmd
    if ![string compare $cmd {}] {  
	set cmd {expr 0}
    }
    bind $e <Key-Return> [format {
	if {![%s]}  {
	    %%W configure -fg black -selectforeground black
	    %%W select clear
	    focus [winfo toplevel [focus]]
#	    focus .
	} else {
	    #mod rbr 2 use new tk_messageBox
	    set curfocus [focus]
	    tk_messageBox -type ok -parent . -icon warning -title "Entry Error"\
		    -message "The data you entered is not valid.\
		    Please correct the entry and try again.\n\n\
		    The data may be the wrong type, e.g. a floating point number when an\
		    integer is expected or letters when numeric data is required.\n\n\
		    The data may be outside of the range of allowed values.\
		    Please check \"Parameter definitions\" in the help system for\
		    any restrictions on this entry."
	    if { [winfo exists $curfocus] } {
		raise [winfo toplevel $curfocus]
		focus $curfocus
	    }
	    #mod rbr 1 use LASSPentry_error proc
#	    LASSPentry_error
#	    tk_dialog .lASSPentryerror Error \
#		    "The data you entered is not valid.  Please correct it \
#		    and try again."  {} 0 Ok
	}
    } $cmd]
	bind $e <Key-KP_Enter> [bind $e <Key-Return>]
	
	
    bind $e <Double-Button-1> {
	%W select from 0
	%W select to end
	break
    }
    bind $e <Button-2> {
	%W configure -fg red -selectforeground red
    }
# Button-2 to Shift-Button-1 for mac and pc - rbr
    bind $e <Shift-Button-1> {
	%W configure -fg red -selectforeground red
    }
    bind $e <Button-3> {
	%W scan mark %x
    }
    bind $e <B3-Motion> {
	%W scan dragto %x
    }

# Button-3 and B3-Motion to Control-Button-1 and Control-B1-Motion -rbr
    bind $e <Control-Button-1> {
	%W scan mark %x
    }
    bind $e <Control-B1-Motion> {
	%W scan dragto %x
    }

    return $e
}


proc LASSPentry_set {e val} {
    $e delete 0 end
    $e insert 0 $val
    $e configure -fg black -selectforeground black
}

proc LASSPentry_error { } {
    if {[winfo exists .lASSPentryerror2]} {
	raise .lASSPentryerror2
    } else {
	toplevel .lASSPentryerror2
	button .lASSPentryerror2.ok -text "Close" \
		-command { destroy .lASSPentryerror2 }
	label .lASSPentryerror2.label\
		-text "The data you entered is not valid.\nPlease correct it\
		and try again." -justify center
	pack .lASSPentryerror2.label -side top -fill x -padx 10 -pady 10
	pack .lASSPentryerror2.ok -side bottom -pady 5
	wm title .lASSPentryerror2 "Error"
	raise .lASSPentryerror2
    }
}