
#####################################################################
# LASSPgraph allows the copying and simple maniulation of the blt_graph 
#   widget.  Thanks to the makers of blt for the excellent blt_graph widget 
#   and for the blt_features.tcl files used.   - Russ Thompson
# $g is a toplevel window name for the new window.
# $source is the window name of the graph widget from which data is stolen.
# To use the routines, call graph_create with the appropriate arguments.

proc graph_create {g source} {
    toplevel $g
    wm minsize $g 100 100

    frame $g.menu 
    button $g.steal -text "steal data" -padx 4 \
	    -command "graph_steal $g $source" 
    button $g.savedata -text "save data..." -padx 4 \
	    -command "graph_save $g data"
    button $g.saveps -text "save postscript..." -padx 4 \
	    -command "graph_save $g ps"
    button $g.config -text "configure"  -padx 4\
	    -command "graph_config $g.graph"
    button $g.close -text "close" -padx 4 \
	    -command "destroy $g"
    pack $g.steal $g.savedata $g.saveps $g.config $g.close -in $g.menu -side left

    blt_graph $g.graph 
    #copy overall configurations
    foreach option [$source configure] {
	set name [lindex $option 0]
	set val [lindex $option 4]
	if {$val != ""} {
	    $g.graph configure $name $val
	}
    }
    #copy configuration options for xaxis and yaxis
    foreach option [$source xaxis configure] {
	catch {$g.graph xaxis configure \
		[lindex $option 0] [lindex $option 4]}
    }
    foreach option [$source yaxis configure] {
	catch {$g.graph yaxis configure \
		[lindex $option 0] [lindex $option 4]}
    }
    #create bindings
    LASSPactivelegend $g.graph
    LASSPgrzoom $g.graph
    LASSPgrcursor $g.graph "format {%.6lg,   %.6lg}"

    graph_steal $g $source

    pack $g.menu -side top -fill x -padx 2 -pady 2
    pack $g.graph -side top -fill both -expand yes
}

proc graph_steal {g source} {
    #find lowest num to create new data set 
    set num 0
    foreach ele [$g.graph element names] {
	regsub -all -- "_" $ele " " ele
	set n [lindex $ele [expr [llength $ele]-1]]
	if {$n>$num} {set num $n}
    }
    incr num 1

    #copy the shown data sets, along with configuration options, etc...
    set showlist [$g.graph element show]
    foreach ele [lsort [$source element show]] {
	$g.graph element create "${ele}_$num"
	foreach option [$source element configure $ele] {
	    catch {$g.graph element configure "${ele}_$num" \
		    [lindex $option 0] [lindex $option 4]}
	}
	$g.graph element configure "${ele}_$num" -label "${ele}_$num"
    }

    #update show list for $g.graph
    foreach ele [lsort [$source element show]] {
	lappend showlist "${ele}_$num"
    }
    $g.graph element show $showlist
}



proc graph_save {g type} {
# mod rbr take first name from list [lindex list 0] rather than whole list
    set name [lindex [LASSPfileselect * {}] 0]
    if {![string compare $name {}]} return 
    if {[file exists $name]} {
	if {[tk_dialog .error {} "File $name already exists.  Overwrite it?" \
		{} 1 Ok Cancel]} return
    }
    if {![string compare $type data]} {
	set file [open $name w]
	foreach elem [$g.graph element names] {
	    set data [lindex [$g.graph element configure $elem -data] 4]
	    set len [llength $data]

	    # some tricky expression substitution is necessary to speed up 
	    # the writing of the data
	    set data [join $data "_"]
	    regsub -all -- {([-+eE0123456789.]+)_([-+eE0123456789.]+)_?} \
		    $data "\\1\t\\2\n" data
	    puts $file $data
	    puts $file "&\n"
	}
	close $file
    } elseif {![string compare $type ps]} {
	$g.graph postscript $name
    }
}

proc graph_toggleshow {g set} {
    set l [$g element show]
    set i [lsearch $l $set]
    if {$i==-1} {
	$g element show [concat $l $set] 
    } else {
	$g element show [lreplace $l $i $i]
    }
}
    

proc graph_config {g} {
    # mod rbr - see if window exists to be configured
    if { ![winfo exists $g] } {
	destroy .graphconfig
	return
    }
    if {[winfo exists .graphconfig]} {
	foreach win [winfo children .graphconfig] {
	    destroy $win
	}
    } else {
	toplevel .graphconfig
	wm title .graphconfig "graph configure"
	wm minsize .graphconfig 200 200
    }
    set w .graphconfig
    
    raise $w
    frame $w.controls
    label $w.controls.l -text "Graph Configure"
    button $w.controls.b -text Close -command "destroy .graphconfig"
    button $w.controls.b2 -text Revert -command "graph_config $g"
    pack $w.controls.l -side left -fill x -expand yes
    pack $w.controls.b2 -side left -fill x -expand yes
    pack $w.controls.b -side left -fill x -expand yes
    pack $w.controls -fill x -expand yes -pady 1
    
    frame $w.title
    label $w.title.l -text Title:
    LASSPentry $w.title.e \
	    [format {catch {%s configure -title [%%W get]}} $g]
    pack $w.title.l  -side left
    pack $w.title.e  -side left -fill x -expand yes
    pack $w.title -fill x -expand yes -pady 1
    
    frame $w.xtitle
    label $w.xtitle.l -text "X Axis Title:"
    LASSPentry $w.xtitle.e \
	    [format {catch {%s xaxis configure -title [%%W get]}} $g]
    pack $w.xtitle.l  -side left
    pack $w.xtitle.e  -side left -fill x -expand yes
    pack $w.xtitle -fill x -expand yes -pady 1
    
    frame $w.ytitle
    label $w.ytitle.l -text "Y Axis Title:"
    LASSPentry $w.ytitle.e \
	    [format {catch {%s yaxis configure -title [%%W get]}} $g]
    pack $w.ytitle.l  -side left
    pack $w.ytitle.e  -side left -fill x -expand yes
    pack $w.ytitle -fill x -expand yes -pady 1
    
    frame $w.font
    label $w.font.l -text "Font:"
    LASSPentry $w.font.e \
	    [format {catch {%s configure -font [%%W get]}} $g]
    pack $w.font.l  -side left
    pack $w.font.e  -side left -fill x -expand yes
    pack $w.font -fill x -expand yes -pady 1
    
    frame $w.xmin
    label $w.xmin.l -text "X Axis Minimum:"
    LASSPentry $w.xmin.e \
	    [format {catch {%s xaxis config -min [%%W get]}} $g]
    pack $w.xmin.l  -side left
    pack $w.xmin.e  -side left -fill x -expand yes
    pack $w.xmin -fill x -expand yes -pady 1
    
    frame $w.xmax
    label $w.xmax.l -text "X Axis Maximum:"
    LASSPentry $w.xmax.e \
	    [format {catch {%s xaxis config -max [%%W get]}} $g]
    pack $w.xmax.l  -side left
    pack $w.xmax.e  -side left -fill x -expand yes
    pack $w.xmax -fill x -expand yes -pady 1
    
    frame $w.ymin
    label $w.ymin.l -text "Y Axis Minimum:"
    LASSPentry $w.ymin.e \
	    [format {catch {%s yaxis config -min [%%W get]}} $g]
    pack $w.ymin.l  -side left
    pack $w.ymin.e  -side left -fill x -expand yes
    pack $w.ymin -fill x -expand yes -pady 1
    
    frame $w.ymax
    label $w.ymax.l -text "Y Axis Maximum:"
    LASSPentry $w.ymax.e \
	    [format {catch {%s yaxis config -max [%%W get]}} $g]
    pack $w.ymax.l  -side left
    pack $w.ymax.e  -side left -fill x -expand yes
    pack $w.ymax -fill x -expand yes -pady 1
    
    frame $w.buttons
    checkbutton $w.showlegend -text "Legend" \
	    -variable $w.showlegend -relief raised \
	    -command "$g legend configure -mapped \[set $w.showlegend]"    
    checkbutton $w.xlog -text "X Log" \
	    -variable $w.xlog -relief raised \
	    -command "$g xaxis configure -min {} -max {} -logscale \[set $w.xlog]"
    checkbutton $w.ylog -text "Y Log" \
	    -variable $w.ylog -relief raised \
	    -command "$g yaxis configure -min {} -max {} -logscale \[set $w.ylog]"
    pack $w.showlegend $w.xlog $w.ylog \
	    -in $w.buttons -side left -fill both -expand yes -pady 1
    pack $w.buttons -fill both -expand yes -pady 1
    ################################################
    frame $w.data -relief ridge -bd 5
    pack $w.data -fill both -expand yes 

    set cmd {
	LASSPselection W.dataset "Data Set:" W.set {
	    LASSPentry_set W.datalabel.e \
		    [lindex [G element configure [set W.set] -label] 4];  
	    LASSPentry_set W.datacolor.e \
		    [lindex [G element configure [set W.set] -fg] 4];
	    set W.datashow \
		    [expr [lsearch [G element show] [LASSPselection_set W.dataset]]!=-1]
    }   }
    regsub -all -- {W} $cmd $w cmd
    regsub -all -- {G} $cmd $g cmd
    foreach name [lsort [$g element names]] {
	lappend cmd [format {%s} $name]
    }
    eval $cmd
    pack $w.dataset -in $w.data -fill both -expand yes -pady 1
    
    frame $w.databuttons
    button $w.datadel -text "delete" -command \
	    "$g element delete \[LASSPselection_set $w.dataset];  \
	    $w.controls.b2 invoke"
    checkbutton $w.datashow -text "show" -variable $w.datashow \
	    -relief raised \
	    -command "graph_toggleshow $g \[LASSPselection_set $w.dataset]"
    pack $w.datashow $w.datadel -in $w.databuttons \
	    -side right -fill both -expand yes
    pack $w.databuttons -in $w.data -fill both -expand yes -pady 1

    frame $w.datalabel
    label $w.datalabel.l -text "Label:"
    LASSPentry $w.datalabel.e [format {catch \
		{%s element config [LASSPselection_set %s.dataset] \
		-label [%%W get]}} \
		$g $w]
    pack $w.datalabel.l -side left
    pack $w.datalabel.e -side right -fill x -expand yes
    pack $w.datalabel -in $w.data -fill both -expand yes -pady 1

    frame $w.datacolor
    label $w.datacolor.l -text "Color:"
    LASSPentry $w.datacolor.e [format {catch \
		{%s element config [LASSPselection_set %s.dataset] -fg [%%W get]}} \
		$g $w]
    pack $w.datacolor.l -side left
    pack $w.datacolor.e -side right -fill x -expand yes
    pack $w.datacolor -in $w.data -fill both -expand yes -pady 1
    
    ################################################
#    foreach set [$g element names] {
#	frame $w.${set}a
#	label $w.${set}a.l -text ${set}:
#	button $w.${set}a.b -text "delete" \
#		-command "$g element delete $set; destroy $w.${set}a $w.${set}b $w.${set}c"
#	checkbutton $w.${set}a.b2 -text "show" -variable $w.${set}a.b2 \
#		-relief raised -command "graph_toggleshow $g $set"
#	if {[lsearch [$g element show] $set]!=-1} {
#	    $w.${set}a.b2 select
#	}

#	pack $w.${set}a.l -side left -fill x -expand yes
#	pack $w.${set}a.b2 -side right -fill both -expand yes
#	pack $w.${set}a.b -side right -fill both -expand yes
	
#	frame $w.${set}b
#	label $w.${set}b.l -text "Label:"
#	LASSPentry $w.${set}b.e \
#		[format {catch {%s element config %s -label [%%W get]}} $g $set]
#	pack $w.${set}b.l -side left
#	pack $w.${set}b.e -side right -fill x -expand yes
#	
#	frame $w.${set}c
#	label $w.${set}c.l -text "Color:"
#	LASSPentry $w.${set}c.e \
#		[format {catch {%s element config %s -fg [%%W get]}} $g $set]
#	pack $w.${set}c.l -side left
#	pack $w.${set}c.e -side right -fill x -expand yes
#	
#	pack $w.${set}a -side top -fill x -expand yes -pady 1
#	pack $w.${set}b -side top -fill x -expand yes -pady 1
#	pack $w.${set}c -side top -fill x -expand yes -pady 1
#    }
#    
    #### initialize objects 
    LASSPentry_set $w.title.e [lindex [$g configure -title] 4]
    LASSPentry_set $w.xtitle.e [lindex [$g xaxis configure -title] 4]
    LASSPentry_set $w.ytitle.e [lindex [$g yaxis configure -title] 4]
    LASSPentry_set $w.font.e [lindex [$g configure -font] 4]
    LASSPentry_set $w.xmin.e [lindex [$g xaxis configure -min] 4]
    LASSPentry_set $w.xmax.e [lindex [$g xaxis configure -max] 4]
    LASSPentry_set $w.ymin.e [lindex [$g yaxis configure -min] 4]
    LASSPentry_set $w.ymax.e [lindex [$g yaxis configure -max] 4]
    if [lindex [$g legend configure -mapped] 4] {
	$w.showlegend select
    } else {
	$w.showlegend deselect
    }
    if [lindex [$g xaxis configure -logscale] 4] {
	$w.xlog select
    } else {
	$w.xlog deselect
    }
    if [lindex [$g yaxis configure -logscale] 4] {
	$w.ylog select
    } else {
	$w.ylog deselect
    }
#    foreach set [$g element names] {
#	LASSPentry_set $w.${set}b.e [lindex [$g element configure $set -label] 4]
#	LASSPentry_set $w.${set}c.e [lindex [$g element configure $set -fg] 4]
#    }


}
    
    

