#mod rbr, initialize SSS_CTRLB1
set SSS_CTRLB1 0

############################################################
# g is the window name of a blt_graph
# fmtcmd is a command that when two coordinates are appended and
#   the command is executed, returns a formatted string that is 
#   placed in the upper left hand corner of the graph.
# for example, fmtcmd could be  "format {%lf %lf}"
proc LASSPgrcursor {g fmtcmd} {
    upvar #0 $g c

    #set up text tag & variables
    set c(cursortag) [$g tag create text "1 1" -anchor nw] 
    set c(fmtcmd) $fmtcmd

    #do bindings for cursor 
    bind $g <Button-1> {
	set %W(text) [eval "[set %W(fmtcmd)] [%W invtransform %x %y]"  ]
	%W tag coords [set %W(cursortag)] \
		"[lindex [%W xaxis limits] 0] [lindex [%W yaxis limits] 1]"
	%W tag configure [set %W(cursortag)] -text [set %W(text)]
    }

#    bind $g <B1-Motion> [bind $g <Button-1>]

    #mod rbr - If started drag with ctrl-b1-motion continue as with 
    # b3 motion, otherwise map to B1 press as before.
    bind $g <B1-Motion> {
	global SSS_CTRLB1
	if { $SSS_CTRLB1 == 0 } {
	    set %W(text) [eval "[set %W(fmtcmd)] [%W invtransform %x %y]"  ]
	    %W tag coords [set %W(cursortag)] \
		"[lindex [%W xaxis limits] 0] [lindex [%W yaxis limits] 1]"
	    %W tag configure [set %W(cursortag)] -text [set %W(text)]
	} else {
	    #get x1,y1 and clip to viewable region on graph
	    set %W(x1) [lindex [%W invtransform %x %y] 0]
	    set %W(y1) [lindex [%W invtransform %x %y] 1]
	    if {[set %W(x1)] < [lindex [%W xaxis limits] 0]} {
		set %W(x1) [lindex [%W xaxis limits] 0]
	    }
	    if {[set %W(x1)] > [lindex [%W xaxis limits] 1]} {
		set %W(x1) [lindex [%W xaxis limits] 1]
	    }
	    if {[set %W(y1)] < [lindex [%W yaxis limits] 0]} {
		set %W(y1) [lindex [%W yaxis limits] 0]
	    }
	    if {[set %W(y1)] > [lindex [%W yaxis limits] 1]} {
		set %W(y1) [lindex [%W yaxis limits] 1]
	    }

	    #redraw the zooming box
	    %W tag coords [set %W(zoomboxtag)] "   \
		    [set %W(x0)] [set %W(y0)]  \
		    [set %W(x1)] [set %W(y0)]  \
		    [set %W(x1)] [set %W(y1)]  \
		    [set %W(x0)] [set %W(y1)]  \
		    [set %W(x0)] [set %W(y0)]"  
	}
    }

    bind $g <ButtonRelease-1> {
	#mod rbr - if started drag with control-B1 do B3 release 
	# otherwise to normal b1 release
	global SSS_CTRLB1
	if { $SSS_CTRLB1 == 1 } {
	    set SSS_CTRLB1 0
	    #get rid of zoom box on screen
	    %W tag coords [set %W(zoomboxtag)] {1 1 1 1}

	    #autoscale if just a click with no motion
	    if {abs(%x-[set %W(screenx0)]) < 3  &&  \
		    abs(%y-[set %W(screeny0)]) < 3} {
		%W xaxis configure -min {} -max {}
		%W yaxis configure -min {} -max {}
		# mod rbr else rather than break
	    } else {

		#get x,y in graph coords
		set %W(x1) [lindex [%W invtransform %x %y] 0]
		set %W(y1) [lindex [%W invtransform %x %y] 1]

		#clip to viewable region on graph
		if {[set %W(x1)] < [lindex [%W xaxis limits] 0]} {
		    set %W(x1) [lindex [%W xaxis limits] 0]
		}
		if {[set %W(x1)] > [lindex [%W xaxis limits] 1]} {
		    set %W(x1) [lindex [%W xaxis limits] 1]
		}
		if {[set %W(y1)] < [lindex [%W yaxis limits] 0]} {
		    set %W(y1) [lindex [%W yaxis limits] 0]
		}
		if {[set %W(y1)] > [lindex [%W yaxis limits] 1]} {
		    set %W(y1) [lindex [%W yaxis limits] 1]
		}

		#make sure that x1>x0, y1>y0
		if {[set %W(x1)] < [set %W(x0)]} {
		    set %W(temp) [set %W(x1)]
		    set %W(x1) [set %W(x0)]
		    set %W(x0) [set %W(temp)]
		}
		if {[set %W(y1)] < [set %W(y0)]} {
		    set %W(temp) [set %W(y1)]
		    set %W(y1) [set %W(y0)]
		    set %W(y0) [set %W(temp)]
		}

		#set new axis limits
		if {[set %W(x0)] != [set %W(x1)]} {
		    %W xaxis configure -min [set %W(x0)] -max [set %W(x1)]
		}
		if {[set %W(y0)] != [set %W(y1)]} {
		    %W yaxis configure -min [set %W(y0)] -max [set %W(y1)]
		}
	    }
	} else { 
	    %W tag configure [set %W(cursortag)] -text {}
	}
    }
}

#############################################################
# taken mostly from the file blt_features
#
proc LASSPactivelegend {g} {
    upvar #0 $g c
    
    set c(lastactive) ""
    bind $g <Motion> {
	set %W(nextactive) [%W legend get @%x,%y]
	if {"[set %W(nextactive)]" != "[set %W(lastactive)]"} {
	    if {"[set %W(lastactive)]" != ""} {
		%W legend deactivate [set %W(lastactive)]
		%W element deactivate [set %W(lastactive)]
	    }
	    if {"[set %W(nextactive)]" != ""} {
		%W legend activate [set %W(nextactive)]
		%W element activate [set %W(nextactive)]
	    }
	    set %W(lastactive) [set %W(nextactive)]
	}
    }
}


###############################################################
#
proc LASSPgrzoom {g} {
    upvar #0 $g c
    set c(zoomboxtag) [$g tag create line] 


    bind $g <Button-3> {
	#save screen coords and get x0,y0 in graph coords
	set %W(screenx0) %x
	set %W(screeny0) %y
	set %W(x0) [lindex [%W invtransform %x %y] 0]
	set %W(y0) [lindex [%W invtransform %x %y] 1]
	
	#clip to viewable region on graph
	if {[set %W(x0)] < [lindex [%W xaxis limits] 0]} {
	    set %W(x0) [lindex [%W xaxis limits] 0]
	}
	if {[set %W(x0)] > [lindex [%W xaxis limits] 1]} {
	    set %W(x0) [lindex [%W xaxis limits] 1]
	}
	if {[set %W(y0)] < [lindex [%W yaxis limits] 0]} {
	    set %W(y0) [lindex [%W yaxis limits] 0]
	}
	if {[set %W(y0)] > [lindex [%W yaxis limits] 1]} {
	    set %W(y0) [lindex [%W yaxis limits] 1]
	}
    }

# Button-3 to Control-Button-1 - rbr
    bind $g <Control-Button-1> {
	#mod rbr
	global SSS_CTRLB1
	set SSS_CTRLB1 1
	#save screen coords and get x0,y0 in graph coords
	set %W(screenx0) %x
	set %W(screeny0) %y
	set %W(x0) [lindex [%W invtransform %x %y] 0]
	set %W(y0) [lindex [%W invtransform %x %y] 1]
	
	#clip to viewable region on graph
	if {[set %W(x0)] < [lindex [%W xaxis limits] 0]} {
	    set %W(x0) [lindex [%W xaxis limits] 0]
	}
	if {[set %W(x0)] > [lindex [%W xaxis limits] 1]} {
	    set %W(x0) [lindex [%W xaxis limits] 1]
	}
	if {[set %W(y0)] < [lindex [%W yaxis limits] 0]} {
	    set %W(y0) [lindex [%W yaxis limits] 0]
	}
	if {[set %W(y0)] > [lindex [%W yaxis limits] 1]} {
	    set %W(y0) [lindex [%W yaxis limits] 1]
	}
    }

    bind $g <B3-Motion> {
	#get x1,y1 and clip to viewable region on graph
	set %W(x1) [lindex [%W invtransform %x %y] 0]
	set %W(y1) [lindex [%W invtransform %x %y] 1]
	if {[set %W(x1)] < [lindex [%W xaxis limits] 0]} {
	    set %W(x1) [lindex [%W xaxis limits] 0]
	}
	if {[set %W(x1)] > [lindex [%W xaxis limits] 1]} {
	    set %W(x1) [lindex [%W xaxis limits] 1]
	}
	if {[set %W(y1)] < [lindex [%W yaxis limits] 0]} {
	    set %W(y1) [lindex [%W yaxis limits] 0]
	}
	if {[set %W(y1)] > [lindex [%W yaxis limits] 1]} {
	    set %W(y1) [lindex [%W yaxis limits] 1]
	}

	#redraw the zooming box
	%W tag coords [set %W(zoomboxtag)] "   \
		[set %W(x0)] [set %W(y0)]  \
		[set %W(x1)] [set %W(y0)]  \
		[set %W(x1)] [set %W(y1)]  \
		[set %W(x0)] [set %W(y1)]  \
		[set %W(x0)] [set %W(y0)]"  
    }

#B3-Motion to Control-B1-Motion - rbr
    bind $g <Control-B1-Motion> {
	#get x1,y1 and clip to viewable region on graph
	set %W(x1) [lindex [%W invtransform %x %y] 0]
	set %W(y1) [lindex [%W invtransform %x %y] 1]
	if {[set %W(x1)] < [lindex [%W xaxis limits] 0]} {
	    set %W(x1) [lindex [%W xaxis limits] 0]
	}
	if {[set %W(x1)] > [lindex [%W xaxis limits] 1]} {
	    set %W(x1) [lindex [%W xaxis limits] 1]
	}
	if {[set %W(y1)] < [lindex [%W yaxis limits] 0]} {
	    set %W(y1) [lindex [%W yaxis limits] 0]
	}
	if {[set %W(y1)] > [lindex [%W yaxis limits] 1]} {
	    set %W(y1) [lindex [%W yaxis limits] 1]
	}

	#redraw the zooming box
	%W tag coords [set %W(zoomboxtag)] "   \
		[set %W(x0)] [set %W(y0)]  \
		[set %W(x1)] [set %W(y0)]  \
		[set %W(x1)] [set %W(y1)]  \
		[set %W(x0)] [set %W(y1)]  \
		[set %W(x0)] [set %W(y0)]"  
    }

    bind $g <ButtonRelease-3> {
	#get rid of zoom box on screen
	%W tag coords [set %W(zoomboxtag)] {1 1 1 1}

	#autoscale if just a click with no motion
	if {abs(%x-[set %W(screenx0)]) < 3  &&  \
	    abs(%y-[set %W(screeny0)]) < 3} {
	    %W xaxis configure -min {} -max {}
	    %W yaxis configure -min {} -max {}
	    #mod rbr use else rather than break
	} else {

	    #get x,y in graph coords
	    set %W(x1) [lindex [%W invtransform %x %y] 0]
	    set %W(y1) [lindex [%W invtransform %x %y] 1]

	    #clip to viewable region on graph
	    if {[set %W(x1)] < [lindex [%W xaxis limits] 0]} {
		set %W(x1) [lindex [%W xaxis limits] 0]
	    }
	    if {[set %W(x1)] > [lindex [%W xaxis limits] 1]} {
		set %W(x1) [lindex [%W xaxis limits] 1]
	    }
	    if {[set %W(y1)] < [lindex [%W yaxis limits] 0]} {
		set %W(y1) [lindex [%W yaxis limits] 0]
	    }
	    if {[set %W(y1)] > [lindex [%W yaxis limits] 1]} {
		set %W(y1) [lindex [%W yaxis limits] 1]
	    }

	    #make sure that x1>x0, y1>y0
	    if {[set %W(x1)] < [set %W(x0)]} {
		set %W(temp) [set %W(x1)]
		set %W(x1) [set %W(x0)]
		set %W(x0) [set %W(temp)]
	    }
	    if {[set %W(y1)] < [set %W(y0)]} {
		set %W(temp) [set %W(y1)]
		set %W(y1) [set %W(y0)]
		set %W(y0) [set %W(temp)]
	    }

	    #set new axis limits
	    if {[set %W(x0)] != [set %W(x1)]} {
		%W xaxis configure -min [set %W(x0)] -max [set %W(x1)]
	    }
	    if {[set %W(y0)] != [set %W(y1)]} {
		%W yaxis configure -min [set %W(y0)] -max [set %W(y1)]
	    }
	}
    }
    
# ButtonRelease-3 to Control-ButtonRelease-1
    bind $g <Control-ButtonRelease-1> {
	#rbr
	global SSS_CTRLB1
	set SSS_CTRLB1 0
	#get rid of zoom box on screen
	%W tag coords [set %W(zoomboxtag)] {1 1 1 1}

	#autoscale if just a click with no motion
	if {abs(%x-[set %W(screenx0)]) < 3  &&  \
	    abs(%y-[set %W(screeny0)]) < 3} {
	    %W xaxis configure -min {} -max {}
	    %W yaxis configure -min {} -max {}
	    #mod rbr else rather than break
	} else {

	    #get x,y in graph coords
	    set %W(x1) [lindex [%W invtransform %x %y] 0]
	    set %W(y1) [lindex [%W invtransform %x %y] 1]
	    
	    #clip to viewable region on graph
	    if {[set %W(x1)] < [lindex [%W xaxis limits] 0]} {
		set %W(x1) [lindex [%W xaxis limits] 0]
	    }
	    if {[set %W(x1)] > [lindex [%W xaxis limits] 1]} {
		set %W(x1) [lindex [%W xaxis limits] 1]
	    }
	    if {[set %W(y1)] < [lindex [%W yaxis limits] 0]} {
		set %W(y1) [lindex [%W yaxis limits] 0]
	    }
	    if {[set %W(y1)] > [lindex [%W yaxis limits] 1]} {
		set %W(y1) [lindex [%W yaxis limits] 1]
	    }
	    
	    #make sure that x1>x0, y1>y0
	    if {[set %W(x1)] < [set %W(x0)]} {
		set %W(temp) [set %W(x1)]
		set %W(x1) [set %W(x0)]
		set %W(x0) [set %W(temp)]
	    }
	    if {[set %W(y1)] < [set %W(y0)]} {
		set %W(temp) [set %W(y1)]
		set %W(y1) [set %W(y0)]
		set %W(y0) [set %W(temp)]
	    }
	    
	    #set new axis limits
	    if {[set %W(x0)] != [set %W(x1)]} {
		%W xaxis configure -min [set %W(x0)] -max [set %W(x1)]
	    }
	    if {[set %W(y0)] != [set %W(y1)]} {
		%W yaxis configure -min [set %W(y0)] -max [set %W(y1)]
	    }
	}
    }
}



