# 
# a lightweight html browser in tcl with a few perks
# 1)  clicking on a link whose href ends in ".pre" causes the associated 
#  file to be sourced, not loaded as html text
# 2)  clicking on "edit" lets you edit the html source for the current file.  
#  clicking on "save" saves the file and takes you back to the rendered view. 
#  If you don't have write permission for the file, then the edit button is 
#  disabled.  
# 3)  setting the url to point to a directory shows a directory view.
# 4)  this browser can only view files.  i.e. no http:// or ftp:// links
# 5)  the # index for referencing a location within a file is not supported. 
# 6)

#mod rbr - global palette set variable
set help_palette 0

proc LASSPhelp2 {w home} {
    upvar #0 $w var

    if {[winfo exists $w]} {
	raise $w
	return
    } 
    
    set var(home) $home
    set var(url) $var(home)
    set var(back) {}
    set var(forward) {}
    set var(edit) 0

    toplevel $w
    frame $w.menu
    button $w.quit -command "destroy $w"  -text close
    button $w.back -text "back" -command "LASSPhelp_back $w"
    button $w.forward -text "forward" -command "LASSPhelp_forward $w"
    button $w.home -text "home" -command "LASSPhelp_home $w"
    button $w.edit -text "edit" -command "LASSPhelp_edit $w"
    button $w.save -text "save" -command "LASSPhelp_save $w"
    button $w.cancel -text "cancel" -command "LASSPhelp_cancel $w"

    entry $w.entry  -width 35
    scrollbar $w.scrollbar  -command "$w.text yview"  -orient v
    text $w.text -height 30 -width 50  -padx 3 -pady 3 \
	    -yscrollcommand "$w.scrollbar set"
    text $w.source -height 30 -width 50  -padx 3 -pady 3 -wrap word  \
	    -yscrollcommand "$w.scrollbar set"
    menubutton $w.size -relief raised -bd 2 -text "size" -menu $w.size.m
    menu $w.size.m -tearoff no
    $w.size.m add radiobutton -label small -value 0   -variable ${w}(size) \
	    -command "HMset_state ${w}.text -size -4; LASSPhelp_render $w \[set ${w}(url)]"
    $w.size.m add radiobutton -label medium -value 4  -variable ${w}(size) \
	    -command "HMset_state ${w}.text -size 2; LASSPhelp_render $w \[set ${w}(url)]"
    $w.size.m add radiobutton -label large -value 12  -variable ${w}(size) \
	    -command "HMset_state ${w}.text -size 4; LASSPhelp_render $w \[set ${w}(url)]"

#mod rbr
    global help_palette
    if { $help_palette == 0 } {
      tk_setPalette #d9d9d9
	set help_palette 1
    }
    
    
    pack $w.back $w.forward $w.home $w.edit $w.size $w.quit -in $w.menu -side left 
    pack $w.entry -in $w.menu -side right -expand 1 -fill both
    pack $w.menu -side top -fill both
    pack $w.scrollbar -side right -expand 0 -fill y
    pack $w.text -side left -fill both -expand 1
    
    # set up some bindings
    bind $w.entry <Return> "LASSPhelp_entry $w"
    bind $w <End> "$w.text yview end"
    bind $w <Home> "$w.text yview 0.0"
    bind $w <Next> "$w.text yview scroll 1 page"
    bind $w <Prior> "$w.text yview scroll -1 page"
    

    HMinit_win $w.text
# mod rbr - replaced with if below
#    HMset_state $w.text -size 0
#    LASSPhelp_render $w $var(url)
    #mod rbr - restore font size to indicated selection
    if { [set var(size)] != {} } {
	if { [set var(size)] == 0 } { ${w}.size.m invoke 0 }
	if { [set var(size)] == 4 } { ${w}.size.m invoke 1 }
	if { [set var(size)] == 12 } { ${w}.size.m invoke 2 }
    } else { 
	set var(size) 4
	${w}.size.m invoke 1
    }
}

# go render a page.  We have to make sure we don't render one page while
# still rendering the previous one.

proc LASSPhelp_render {w file} {
    upvar #0 $w var
    
    
    if {[catch {set fd [open $file]} msg]} {
	set result "<title>Bad file $file</title> \n\
		<h1>Error reading $file</h1><p>  \n\
		$msg<hr>"
    } else {
	if {[file isdirectory $file]} {
	    close $fd
	    if {[file tail $file]!=""} {
		append file "/"
	    }
	    set var(url) $file
	    set result "<title>Index of $file</title>  \n\
		    <h1> Index of $file </h1><p> "
	    foreach f [lsort [glob .. ${file}/*]] {
		set f [file tail $f]
		append result "<a href=${f}>${f}</a><br>\n"
	    }
	} else {
	    set result [read $fd]
	    close $fd
	}

	if {[file writable $file]} {
	    $w.edit configure -state normal
	} else {
	    $w.edit configure -state disabled
	}
    }
    
    $w.entry delete 0 end
    $w.entry insert 0 $file

    $w.text configure -state normal
    HMreset_win $w.text

    $w.source delete 0.0 end
    $w.source insert 0.0 $result

    HMparse_html $result "HMrender $w.text"
    HMset_state $w.text -stop 1		;# stop rendering previous page if busy
    $w.text configure -state disabled
}


proc LASSPhelp_entry {w}  {
    upvar #0 $w var
    
    set file [$w.entry get]
    if {![file exists $file] } {
	if {[file writable [file dirname $file]]} {
	    set cancel [tk_dialog .temp {} \
		    "The file specified doesn't exist.  Would you like to create it?" \
		    {} 0 Ok Cancel]
	    if {$cancel} {
		$w.entry delete 0 end
		$w.entry insert 0 $var(url)
		return
	    } else {
		set fd [open $file w]
		set result \
			"<title>$file</title> \n\
			<h1>my new file: $file</h1> "
		puts $fd $result
		close $fd
	    }
	} else {
	    tk_dialog .temp {} \
		    "Error: The file specified does not exist, and the file permissions do not allow you to create one." {} 0 Ok
	    $w.entry delete 0 end
	    $w.entry insert 0 $var(url)
	    return
	}
    }

    set var(forward) {}
    lappend var(back) $var(url)
    set var(url) $file
    LASSPhelp_render $w $var(url)
}


proc LASSPhelp_back {w}  {
    upvar #0 $w var
    
    set length [llength $var(back)]
    if {$length>0} {
	lappend var(forward) $var(url)
	set var(url) [lindex $var(back) [expr $length -1]]
	set var(back) [lrange $var(back) 0 [expr $length -2]]
	LASSPhelp_render $w $var(url)
    }    
}


proc LASSPhelp_forward {w}  {
    upvar #0 $w var

    set length [llength $var(forward)]
    if {$length>0} {
	lappend var(back) $var(url)
	set var(url) [lindex $var(forward) [expr $length -1]]
	set var(forward) [lrange $var(forward) 0 [expr $length -2]]
	LASSPhelp_render $w $var(url)
    }    
}

proc LASSPhelp_home {w}  {
    upvar #0 $w var
    
    lappend var(back) $var(url)
    set var(url) $var(home)
    set var(forward) {}
    LASSPhelp_render $w $var(url)
}

proc LASSPhelp_edit {w}  {
    upvar #0 $w var
     
    pack forget $w.text
    pack forget $w.back $w.forward $w.home $w.edit $w.size $w.quit
    pack $w.source -side left -fill both -expand 1
    pack $w.save $w.cancel $w.quit -side left -in $w.menu
    $w.scrollbar configure -command "$w.source yview"
    $w.entry configure -state disabled
    set var(edit) 1
}

proc LASSPhelp_save {w}  {
    upvar #0 $w var

    if {!([file isfile $var(url)] && [file writable $var(url)])} {
	set file [LASSPfileselect * [file tail $var(url)]]
	if {[llength $file]==0} {
	    return
	}
	if {[llength $file]>1} {
	    tk_dialog .temp {} "Too many file names selected" {} 0 Ok
	    return
	}
	set var(url) $file
    }
    
    if {[catch {set id [open $var(url) w]} msg]} {
	tk_dialog .temp {} "error opening file: $msg" {} 0 Ok
	return
    } else {
	puts $id [$w.source get 0.0 end]
	close $id
    }
    LASSPhelp_render $w $var(url)
    
    pack forget $w.source
    pack $w.text -side left -fill both -expand 1
    pack forget $w.cancel $w.save $w.quit
    pack $w.back $w.forward $w.home $w.edit $w.size $w.quit -in $w.menu -side left   
    $w.scrollbar configure -command "$w.text yview"
    $w.entry configure -state normal
    set var(edit) 0
}

proc LASSPhelp_cancel {w}  {
    upvar #0 $w var

    pack forget $w.source
    pack $w.text -side left -fill both -expand 1
    pack forget $w.cancel $w.save $w.quit
    pack $w.back $w.forward $w.home $w.edit $w.size $w.quit -in $w.menu -side left   
    $w.scrollbar configure -command "$w.text yview"
    $w.entry configure -state normal
    set var(edit) 0
}



# Override the library link-callback routine for the sample app.
# It only handles the simple cases

proc HMlink_callback {win href} {
    set w [string range $win 0 [expr [string last ".text" $win]-1]]
    upvar #0 $w var

    if {[string match "/*" $href]} {
	set file $href
    } else {
# rbr
	set file [file join [file dirname $var(url)] $href]
    }
    
    if {[file extension $file]==".pre"}  {
        # load the preset here by sourcing file at global level
        # rbr and again to set preset check button - SSS_PRESETS set in 
	# presets.tcl Assumes only one such menu per program.
	global SSS_PRESETS
        uplevel #0 "source {$file};set {$SSS_PRESETS} {$file}"
	return
    }

    lappend var(back) $var(url)
    set var(forward) {}
    set var(url) $file
    LASSPhelp_render $w $var(url)
}

# supply an image callback function
# Read in an image if we don't already have one
# callback to library for display

proc HMset_image {win handle src} {
    set w [string range $win 0 [expr [string last ".text" $win]-1]]
    upvar #0 $w var

    if {[string match /* $src]} {
	set image $src
    } else {
# rbr	set image [file dirname $var(url)]/$src
	set image [file join [file dirname $var(url)] $src]
    }
    if {[string first " $image " " [image names] "] >= 0} {
	HMgot_image $handle $image
    } else {
	catch {image create photo $image -file $image} image
	HMgot_image $handle $image
    }
}

# Lets invent a new HTML tag, just for fun.
# Change the color of the text. Use html tags of the form:
# <color value=blue> ... </color>
# We can invent a new tag for the display stack.  If it starts with "T"
# it will automatically get mapped directly to a text widget tag.

proc HMtag_color {win param text} {
    upvar #0 HM$win var
    set value bad_color
    HMextract_param $param value
    $win tag configure $value -foreground $value
    HMstack $win "" "Tcolor $value"
}

proc HMtag_/color {win param text} {
    upvar #0 HM$win var
    set value bad_color
    HMstack $win / "Tcolor {}"
}

