# b is the name of the preset button
# p is the name of a presetsave procedure: this procedure must write 
#  out to the file all the tcl code to restore the state of the program 
#  when it is sourced.
# names is a list of filenames as returned from fileselect
proc LASSPpresets_save {b p names} {
    if {0==[llength $names]} {
	return
    } elseif {[llength $names]>1} {
	tk_dialog .temp {} "Can't save to more than one filename" {} 0 "Ok"
	return
    } else {
	set name [lindex $names 0]
    }
    if {[file exists $name]} {
	if {[tk_dialog .temp {} "File $name already exists.  Overwrite it?" \
		{} 1 Ok Cancel]} return
    }
    
    set file [open $name w]
    #### add protection so that other programs can't use this preset
    global SSS_PROG
    set cmd {if {"$SSS_PROG"!="name"} {tk_dialog .temp {} "This preset is meant for the program 'name', and cannot be loaded by '$SSS_PROG'" {} 0 "Ok"; return}}
    regsub -all -- "name" $cmd $SSS_PROG cmd
    puts $file $cmd

    #### add code to update variable $b (global) to reflect name of preset
    # mod rbr - No longer need to do this
    # puts $file "set $b \"$name\""

    #### call presetsave procedure to restore state of program
    $p $file
    close $file
    
    set label [file rootname [file tail $name]]
    if ![catch {$b.m index $label}] { 
	$b.m delete [$b.m index [file rootname [file tail $name]]] 
    }
# surround file or directory names in {} for spaces in mac files - rbr
    $b.m add radiobutton -variable $b -value $name -label $label \
	    -command  "source {$name}"
	 global $b
	 set $b $name
}

# b is name of preset button
# names is a list of preset filenames to be inserted into the preset menu
# if source is "-source" then the last preset will actually be loaded
proc LASSPpresets_load {b names source} {

	 if {0==[llength $names]} return

	 #check for existence and include in menu
	 foreach name $names {
	if {![file exists $name]} {
		 if {[tk_dialog .temp {} "File $name does not exist." \
			 {} 0 Ok]} return
	} else {
		 set loadme $name
		 set label [file rootname [file tail $name]]
		 if ![catch {$b.m index $label}] {
		$b.m delete [$b.m index $label]
		 }
# surround file or directory names in {} for spaces in mac files - rbr
# mod rbr 1/29/97 add set to command for name and loadme below
		 $b.m add radiobutton -variable $b -value $name -label $label \
			 -command  "source {$name};set {$b} {$name}"
	}
	 }
	 #actually load just the last one, maybe
	 if {$source=="-source"} {
# surround file or directory names in {} for spaces in mac files - rbr
	uplevel #0 "source {$loadme};set {$b} {$loadme}"
	 }
}


# b is the name of a preset button to be created
# p is the name of a presetsave procedure
proc LASSPpresets {b p} {
	 menubutton $b -menu $b.m -text "presets" -relief raised
	 menu $b.m
	 #mod rbr - Need a global for preset menu - "there can be only one."
	 global SSS_PRESETS
	 set SSS_PRESETS $b
	 #end mod rbr
	 $b.m add command -label "load..."  -command   \
		 "LASSPpresets_load $b \[LASSPfileselect *.pre {}] -source"
	 $b.m add command -label "save..."  -command   \
		 "LASSPpresets_save $b $p \[LASSPfileselect *.pre \[file tail \[set $b]]]"
	 $b.m add separator

	 set names {}
	 #mod rbr - replace *.pre with {*.pre,*.PRE}
	 foreach name [lsort [glob -nocomplain *.{pre,PRE}]] {
# use file join instead of / -rbr
# was	lappend names "[pwd]/$name"
	lappend names "[file join [pwd] $name]"
	 }
	 LASSPpresets_load $b $names -nosource
	 global $b
	 set $b ""
}

# Added by spt for dynamic font changing.
# here, a trick has to be played as the seperator menu item
# does not have a font property, we call the first two for
# "load" and "save" manually and then the rest in the loop.

proc LASSPpresets_configure {b args} {
	 set len [llength $args]
	 for {set i 0} {$i<$len} {incr i +2} {
		set var [lindex $args $i]
		if {$i+1 < $len} {
			set val [lindex $args [expr $i+1]]
			if {$var=="-font"} { $b configure -font $val

										# The first two elements.
										$b.m entryconfigure 1 -font $val
										$b.m entryconfigure 2 -font $val
										set ind_end [$b.m index end]

										for {set j 4} {$j <= $ind_end} {incr j +1} {
											$b.m entryconfigure $j -font $val
										}
									 }
			}
	}
}

