
# rbr custom bindings for mac
# shift-b1 is b2, control-b1 is b3
bind Scrollbar <Shift-B1-Motion> {
	 tkScrollDrag %W %x %y
}
bind Scrollbar <Shift-1> {
	 tkScrollButton2Down %W %x %y
}
bind Scrollbar <Shift-ButtonRelease-1> {
	 tkScrollButtonUp %W %x %y
}
bind Scrollbar <Control-Shift-1> {
	 tkScrollTopBottom %W %x %y
}


#################### public functions: anyone may call these ####################

proc LASSPscale2 {name args} {
	 upvar #0 $name s

	 #general characteristics
	 set s(-var) ""
	 set s(-initial) 0.0
	 set s(-text) $name
	 set s(-command) {expr 0}

	 #absolute limitations
	 set s(-type) real
	 set s(-test) {1}

	 #scrollbar characteristics
	 set s(-scrollinc) {}
	 set s(-scrollmin) 0
	 set s(-scrollmax) 1
	 set s(-scrolltype) linear

	 frame $name -bd 2 -relief raised
	 LASSPentry $name.e "LASSPscale2_set $name \[%W get]" -relief sunken -width 8
	 #mod rbr - relief sunken looks better in MS Windows -relief flat gets lost
	 scrollbar $name.s -orient horiz -width 10 -relief sunken -highlightthickness 0 -command "LASSPscale2_scroll $name"
	 label $name.l
	 button $name.b -padx 2 -pady 0 -text {} -command "LASSPscale2_configurebox $name"

	 #handle configuration options in a cheap way
	 eval [concat LASSPscale2_configure $name $args]

	 if {$s(-scrollinc)==""} {
		if {$s(-type) == "integer"} {
		 	set s(-scrollinc) 1
		} elseif {$s(-type) == "real"} {
		 	set s(-scrollinc) .01
		}
	 }
	 set s(-font) [lindex [$name.e configure -font] 4]

	 LASSPscale2_set -nocommand $name $s(-initial)

	 pack $name.s -side bottom -fill x
	 pack $name.l -side left
	 pack $name.b -side right -padx 2
	 pack $name.e -side right -padx 2
}

##################################################################3
proc LASSPscale2_get {name} {
	 return [$name.e get]
}


##################################################################3
#format: LASSPscale2_set ?-nocommand? name ?value?
#if -nocommand is specified the value and display are updated without executing the command
#if no value is specified then the current value is used.  Useful for updating the display.
proc LASSPscale2_set {args} {
	 if {[lindex $args 0]=="-nocommand"} {
	set nocommand 1
	set args [lreplace $args 0 0]
	 } else {
	set nocommand 0
	 }
	 set name [lindex $args 0]
	 if {[llength $args] == 1} {
	set value [$name.e get]
	 } elseif {[llength $args] == 2} {
	set value [lindex $args 1]
	 }

	 upvar #0 $name s
	 global $s(-var)

	 if {$s(-type) == "integer" } {
	set typeexpr {~%v}
	 } elseif {$s(-type) == "real"} {
	set typeexpr {-%v}
	 } else {
	set typeexpr {0}
	 }

	 set tst [format {[catch {expr %s}] || ![expr %s] } \
		 $typeexpr $s(-test) ]
	 regsub -all -- {%v} $tst "($value)" tst

	 ## do if value is acceptable...
	 if "![expr $tst]" {
	if {$s(-scrolltype)=="linear"} {
		 LASSPentry_set $name.e $value
		 set width [expr round( ($s(-scrollmax)-$s(-scrollmin)) / $s(-scrollinc) )]
		 set s2_pos [expr round( ($value-$s(-scrollmin)) / $s(-scrollinc) )]
	} elseif {$s(-scrolltype)=="log"} {
		 LASSPentry_set $name.e [format {%.2le} $value]
		 set temp [expr ($s(-scrollmax)-$s(-scrollmin)) / $s(-scrollinc)]
		 if {$temp < 1e9} {
		set width [expr round($temp)]
		 } else {
		set width $temp
		 }
		 set temp [expr $width * log(1.0*$value/$s(-scrollmin))    \
			 / log(1.0*$s(-scrollmax)/$s(-scrollmin))]
		 if {$temp<1e9} {
		set s2_pos [ expr round($temp)]
		 } else {
		set s2_pos $temp
		 }
	}
	#set thumb to be 10% of full scale
	set thumb [expr $width/11]
	# mod rbr lose thumb in first expr.
	$name.s set [expr $width] [expr $thumb+2] $s2_pos [expr $s2_pos+$thumb]

	if {$s(-var)!=""} {
		 global $s(-var)
		 set $s(-var) $value
	}

	if {!$nocommand} {
		 set cmd $s(-command)
		 regsub -all -- {%v} $cmd "($value)" cmd
		 eval $cmd
	}
	return 0
	 } else {
	return 1
	 }
}

####################### LASSPscale2_configure ###########################
# valid options are:
#  -var         specifies a global variable to contain the contents of scale
#  -initial     specifies the inital value of scale
#  -text        specifies the text to display on the scale
#  -font        specifies the font for the scale display
#  -command     specifies a script to execute when the scale value is changed.
#                this is only executed if the value is accepted by the test condition.
#                %v in the string is replaced with the value of the scale
#  -type        either integer or real
#  -test        specifies an expression to evaluate to check if a value is acceptable.
#                1=ok, 0=not ok.  %v is substituted as with -command
#  -scrollmin   specifies the minimum value of the scrollbar
#  -scrollmax   specifies the maximum value of the scrollbar
#  -scrollinc   specifies the increment in which the scrollbar steps
#  -scrolltype  either linear or log
proc LASSPscale2_configure {name args} {
	 upvar #0 $name s

	 set len [llength $args]
	 for {set i 0} {$i<$len} {incr i +2} {
	set var [lindex $args $i]
	if {$i+1 < $len} {
		 set val [lindex $args [expr $i+1]]
		 if {$var=="-text"} {$name.l configure -text $val}
		 if {$var=="-font"} {
		 	$name.e configure -font $val
		 	$name.l configure -font $val
		 }
		 set  s($var) $val
	} else {
		 return $s([lindex $args $i])
	}
    }
    LASSPscale2_set -nocommand $name 
}


#################### non-public functions ##############################
proc LASSPscale2_scroll {name number} {
    upvar #0 $name s

    if {$s(-scrolltype)=="linear"} {
	set value [expr $number*$s(-scrollinc) + $s(-scrollmin)]

	set s2_pos [expr round( ($value-$s(-scrollmin)) / $s(-scrollinc) )]
    } elseif {$s(-scrolltype)=="log"} {
	set temp [expr ($s(-scrollmax)-$s(-scrollmin)) / $s(-scrollinc)]
	if {$temp < 1e9} {
	    set width [expr round($temp)]
	} else {
	    set width $temp
	}
	set value [expr $s(-scrollmin)  \
		* pow( 1.0*$s(-scrollmax)/$s(-scrollmin) ,  1.0*$number/$width) ]
	set temp [expr ($value-$s(-scrollmin))/$s(-scrollinc)]
	if {$temp<1e9} {
	    set value [expr round($temp) * $s(-scrollinc) + $s(-scrollmin)]
	} else {
	    set value [expr $temp * $s(-scrollinc) + $s(-scrollmin)]
	}
    }
    LASSPscale2_set $name $value
}

proc LASSPscale2_configtoggle {w name} {
    upvar #0 $name s

    if {$s(-scrolltype)=="linear"} {
	set s(-scrolltype) "log"
	$w.type.b configure -text "log"
    } else {
	set s(-scrolltype) "linear"
	$w.type.b configure -text "linear"
    }
}    

proc LASSPscale2_configurebox {name} {
    upvar #0 $name s

    if {[winfo exists .scaleconfig]} {
	foreach child [winfo children .scaleconfig] {
	    destroy $child
	}
    } else {
	toplevel .scaleconfig
	wm title .scaleconfig "scale configure"
	wm minsize .scaleconfig 100 100
    }
    set w .scaleconfig
# mod rbr
    raise $w
    
    frame $w.controls
    label $w.controls.l -text "Configure: $s(-text)"
    button $w.controls.b -text Close -command "destroy .scaleconfig"
    pack $w.controls.l -side left -fill x -expand yes
    pack $w.controls.b -side left -fill x -expand yes
    pack $w.controls -fill x -expand yes -pady 1
    
    if {$s(-type) == "integer" } {
	set typeexpr {~[%W get]}
    } elseif {$s(-type) == "real"} {
	set typeexpr {-[%W get]}
    } else {
	set typeexpr {0}
    }

    frame $w.min
    label $w.min.l -text "Minimum:"
    LASSPentry $w.min.e \
	    [format {expr {  \
	    [catch {expr %s}] ||  \
	    [catch {set %s [%%W get]; LASSPscale2_set %s}] }}   \
	    $typeexpr ${name}(-scrollmin) $name ]
    pack $w.min.l  -side left
    pack $w.min.e  -side left -fill x -expand yes
    pack $w.min -fill x -expand yes -pady 1
    LASSPentry_set $w.min.e $s(-scrollmin)
    
    frame $w.max
    label $w.max.l -text "Maximum:"
    LASSPentry $w.max.e \
	    [format {expr {  \
	    [catch {expr %s}] ||  \
	    [catch {set %s [%%W get]; LASSPscale2_set %s}] }}   \
	    $typeexpr ${name}(-scrollmax) $name ]
    pack $w.max.l  -side left
    pack $w.max.e  -side left -fill x -expand yes
    pack $w.max -fill x -expand yes -pady 1
    LASSPentry_set $w.max.e $s(-scrollmax)
    
    frame $w.inc
    label $w.inc.l -text "Increment:"
    LASSPentry $w.inc.e \
	    [format {expr {  \
	    [catch {expr %s}] ||  \
	    [catch {set %s [%%W get]; LASSPscale2_set %s}] }}   \
	    $typeexpr ${name}(-scrollinc) $name ]
    pack $w.inc.l  -side left
    pack $w.inc.e  -side left -fill x -expand yes
    pack $w.inc -fill x -expand yes -pady 1
    LASSPentry_set $w.inc.e $s(-scrollinc)

# rbr remove font configuration.    
#    frame $w.font
#    label $w.font.l -text "Font:"
#    LASSPentry $w.font.e \
#	    [format {catch {LASSPscale2_configure %s -font [%%W get]}} $name]
#    pack $w.font.l  -side left
#    pack $w.font.e  -side left -fill x -expand yes
#    pack $w.font -fill x -expand yes -pady 1
#    LASSPentry_set $w.font.e [LASSPscale2_configure $name -font]
    
    frame $w.type
    label $w.type.l -text "Type:"
    button $w.type.b -command \
	    "LASSPscale2_configtoggle $w $name;  \
	    LASSPscale2_set $name"
    pack $w.type.l  -side left
    pack $w.type.b  -side left -fill x -expand yes
    pack $w.type -fill x -expand yes -pady 1
    $w.type.b configure -text $s(-scrolltype)
}





