

proc LASSPzoom_Button {g x y args} {
    upvar #0 $g c
    set xy [$g invtransform $x $y]
    set c(screenx0) $x
    set c(screeny0) $y
    set c(x0) [lindex $xy 0]
    set c(y0) [lindex $xy 1]
    set c(x1) {}
    set c(y1) {}
    set c(xmin) [lindex [$g xaxis limits] 0]
    set c(xmax) [lindex [$g xaxis limits] 1]
    set c(ymin) [lindex [$g yaxis limits] 0]
    set c(ymax) [lindex [$g yaxis limits] 1]

    set c(texttag) [$g tag create text "$c(xmin) $c(ymax)" -anchor nw \
	    -text [format {%lf, %lf} $c(x0) $c(y0)]]

    if {$args == "-nozoom"} {
	return
    }

    if {$c(x0)<$c(xmin) || $c(x0)>$c(xmax)} {
	set c(x0) $c(xmin)
    }
    if {$c(y0)<$c(ymin) || $c(y0)>$c(ymax)} {
	set c(y0) $c(ymin)
    }
    set c(linetag) [$g tag create line] 
}

proc LASSPzoom_Motion {g x y args} {
    upvar #0 $g c
    
    set xy [$g invtransform $x $y]
    set c(x1) [lindex $xy 0]
    set c(y1) [lindex $xy 1] 
    catch {
	$g tag configure $c(texttag) -text [format {%lf, %lf} $c(x1) $c(y1) ]
    }

    if {$args == "-nozoom"} {
	return
    }

    if {abs($x-$c(screenx0))<3 && abs($y-$c(screeny0))<3} {
	return
    }

    if {$c(x1)<$c(xmin) || $c(x1)>$c(xmax)} {
	set c(x1) $c(xmax)
    }
    if {$c(y1)<$c(ymin) || $c(y1)>$c(ymax)} {
	set c(y1) $c(ymax)
    }
    catch {
	$g tag coords $c(linetag) "$c(x0) $c(y0) $c(x0) $c(y1) \
		$c(x1) $c(y1) $c(x1) $c(y0) $c(x0) $c(y0)"
    }
}


proc LASSPzoom_Release {g x y args} {
    upvar #0 $g c

    catch {$g tag delete $c(texttag)}
    catch {$g tag delete $c(linetag)}

    if {$args == "-nozoom"} {
	return
    }

    if {$c(x1)=={}} {
	$g xaxis configure -min {} -max {}
	$g yaxis configure -min {} -max {}
	return
    }
	
    if {abs($x-$c(screenx0))<3 && abs($y-$c(screeny0))<3} {
	return
    }

    if {$c(x1)<$c(x0)} {
	set temp $c(x0)
	set c(x0) $c(x1)
	set c(x1) $temp
    }
    if {$c(y1)<$c(y0)} {
	set temp $c(y0)
	set c(y0) $c(y1)
	set c(y1) $temp
    }
    
    if {$c(x1)>$c(xmin) && $c(x1)<$c(xmax) && $c(x0)<$c(x1)} {
	$g xaxis configure -min $c(x0) -max $c(x1)
    }
    if {$c(y1)>$c(ymin) && $c(y1)<$c(ymax) && $c(y0)<$c(y1)} {
	$g yaxis configure -min $c(y0) -max $c(y1)
    }
}
    

proc LASSPzoom {g} {
    bind $g <Button-1> "LASSPzoom_Button $g %x %y -nozoom"
    bind $g <B1-Motion> "LASSPzoom_Motion $g %x %y -nozoom"
    bind $g <ButtonRelease-1> "LASSPzoom_Release $g %x %y -nozoom"
    bind $g <Button-3> "LASSPzoom_Button $g %x %y"
    bind $g <B3-Motion> "LASSPzoom_Motion $g %x %y"
    bind $g <ButtonRelease-3> "LASSPzoom_Release $g %x %y"
#Add control button-1 to emulate button-3 for mac - rbr
    bind $g <Control-Button-1> "LASSPzoom_Button $g %x %y"
    bind $g <Control-B1-Motion> "LASSPzoom_Motion $g %x %y"
    bind $g <Control-ButtonRelease-1> "LASSPzoom_Release $g %x %y"
}
